local lg, lm = love.graphics, love.mouse

local a, b, sel, test, hint

function love.load()
    test = require 'rbox'
    a = test.new(200, 300, 100, 100, 0)
    b = test.new(600, 400, 200, 150, 0)
    
    hint = 'mouse button 1 - select and move object\nmouse button 2 - select and project object\nmouse wheel - rotate selected object'
    
end

function love.draw()
    if test.overlaps(a, b) then
        lg.setColor(1, 0, 0)
    else
        lg.setColor(0, 1, 1)
    end
    
    a:draw()
    b:draw()
    
    if sel then
        local cx, cy = sel:getPos()
        local other = (a == sel) and b or a
        local sum = test.RBoxSum(other, sel)
        lg.setColor(1, 1, 0)
        
        local j
        for i = 1, #sum do
            j = i < #sum and i + 1 or 1
            lg.line(sum[i].x, sum[i].y, sum[j].x, sum[j].y)
        end
        
        if lm.isDown(1) then
            lg.circle('fill', cx, cy, 5, 6)
        elseif lm.isDown(2) then
            local goalX, goalY = lm.getPosition()
            lg.line(cx, cy, goalX, goalY)
            
            local col = test.collides(sel, other, goalX, goalY)
            if col then
                lg.setColor(0, 1, 0)
                sel:draw(col.dx, col.dy)
                lg.setColor(1, 0, 0)
                lg.circle('fill', col.px, col.py, 5, 6)
                lg.line(col.px, col.py, col.px + col.nx * 30, col.py + col.ny*30)
            end
        end
    end
    
    lg.setColor(0, 1, 0)
    lg.print(hint)
end

function love.mousepressed(x, y, button)
    if button == 1 or button == 2 then
        if a:contains(x, y) then
            sel = a
        elseif b:contains(x, y) then
            sel = b
        end
    end
end

function love.mousereleased(x, y, button)
    if (button == 1 or button == 2) and sel then
        sel = nil
    end
end

function love.mousemoved(x, y, dx, dy)
    if lm.isDown(1) and sel then
        sel:move(dx, dy)
    end
end

local rot_speed = math.rad(5)
function love.wheelmoved(x, y)
    if y ~= 0 and sel then
        sel:rotate(y * rot_speed)
    end
end

function love.keypressed(key)
    if key == 'escape' then
        love.event.quit()
    end
end